//$Id: management.js,v 1.9 2008/10/18 18:25:04 silviogutierrez Exp $

var presentationManager = presentationManager || {};


$(document).ready(function() {

  //Start up the manager.
  presentationManager.init();
  
});



/**
 * Sets the cursor, binds the keys, and initializes globals.
 *
 */
presentationManager.init = function() {
  
  //Load settings.
  presentationManager.index = Drupal.settings.presentationManager.index; 
  presentationManager.pictures = Drupal.settings.presentationManager.pictures;
  presentationManager.captions = Drupal.settings.presentationManager.captions;  
  presentationManager.settings = Drupal.settings.presentationManager.settings;

  
  presentationManager.current = 0;
  presentationManager.count = Drupal.settings.presentationManager.count;  
  
  presentationManager.initKeys();
  
  //Initialize tabbing functionality.
  $('.presentation-caption-field').focus(function () {
    
    presentationManager.focus(presentationManager.getID(this.id));
  });
  
  
  //Initialize thumbnail clicking functionality.
  $('.presentation-thumbnail-field').click(function () {
  
    $('#presentation-caption-field-' + presentationManager.getID(this.id)).get(0).focus();
  
  });
  
  //Create a delete object.
  presentationManager.toDelete = {};
  
  
  //Initialize deletion functionality.
  $('.presentation-delete-field').click(function () {
    
    var pid = presentationManager.getID(this.id);
    presentationManager.flagDeletion(pid);
    
    //Switch the buttons.
    $(this).hide();
    $('#presentation-restore-field-' + pid).show();
    
    return false;
  });
  
  
  //Initialize restore functionality.
  $('.presentation-restore-field').click(function () {
    
    var pid = presentationManager.getID(this.id);
    presentationManager.unflagDeletion(pid);
    
    //Switch the buttons.
    $(this).hide();
    $('#presentation-delete-field-' + pid).show();
    
    return false;
  }); 
  
  
  presentationManager._initSort();
  presentationManager._initCaptions();
  
  
  //Enable save button.
  $('.presentation-management-save').click(function () {
    presentationManager.save();
    return false;
  });
  
  
  var first = presentationManager.index[0];
  
  
  //Focus on the first field.
  $('#presentation-caption-field-' + first).get(0).focus();
  
  presentationManager.current = 0;
  
  
}





/** 
 * Initializes sorting links.
 *
 */
presentationManager._initSort = function () {
  $('.presentation-up-field').click(function () {
    presentationManager.moveUp(presentationManager.getID(this.id));
    return false;  
  });
  
  $('.presentation-down-field').click(function () {
    presentationManager.moveDown(presentationManager.getID(this.id));
    return false;   
  });

  $('.presentation-top-field').click(function () {
    presentationManager.moveTop(presentationManager.getID(this.id));
    return false;   
  });

  
  $('.presentation-bottom-field').click(function () {
    presentationManager.moveBottom(presentationManager.getID(this.id));
    return false;   
  });
}


/**
 * Populate the caption fields.
 *
 */
presentationManager._initCaptions = function () {
  $('.presentation-caption-field').each(function () {
  
    var pid = presentationManager.getID(this.id);
    
    $('#presentation-caption-field-' + pid).val(presentationManager.captions[pid]);
  });


}



/************************** Key Shortcuts *****************************/



/**
 * Bind special keys to their new functions.
 *
 */
presentationManager.initKeys = function () {

  $(document).keydown(function(key) {
    var pid = presentationManager.index[presentationManager.current];
  
    if (key.keyCode) {
      
      
      switch(key.keyCode) {
        case 9: { // Tab
          return presentationManager.tab(key.shiftKey);   
        }
        case 38: { // Up
          if (key.ctrlKey && presentationManager.settings.allowSorting) {
            presentationManager.moveUp(pid);
            presentationManager.focus(pid);
            return false;
          }        
        }       
        case 40: { // Down
          if (key.ctrlKey && presentationManager.settings.allowSorting) {
            presentationManager.moveDown(pid);
            presentationManager.focus(pid);
            return false;
          }
        }
        case 83: { // Letter S
          if (key.ctrlKey && presentationManager.settings.allowSorting) {
            presentationManager.save();
            return false;
          }        
        
        }

      
      }
      
    }
    else if(key.charCode) {
    
    
    
    }
    else {
      
    }
    
    return true;
    

  });
}



/**
 * Switches to the next available picture.
 *
 */
presentationManager.tab = function (shift) {
  
  if (presentationManager.count == 0) {
    return false;
  }
  
  
  var newIndex = shift ? presentationManager.current - 1 : presentationManager.current + 1;
  
  if (newIndex == presentationManager.index.length) {
    newIndex = 0;
  }
  else if (newIndex == -1) {
    newIndex = presentationManager.index.length - 1;
  }  

  var target = presentationManager.index[newIndex];

  $('#presentation-caption-field-' + target).get(0).focus();
  
  return false;
}





/****************************** Save **********************************/


/**
 * Save all the changes with an AJAX request.
 *
 */
presentationManager.save = function () {


  //Delete pictures we don't need to process.
  presentationManager.deletePictures();
  
  
  //Prepare a JSON containing all the data to save.
  var data = {};
 


  for (var pid in presentationManager.toDelete) {
    data[pid] = 'delete' + '|#|';   
  }
  
  
  
  
  
  for (var pid in presentationManager.pictures) {

    var index = presentationManager.pictures[pid];
    var caption = $('#presentation-caption-field-' + pid).val();    
    
    data[pid] = "sort" + "|#|" + index + "|#|" + caption; 
  }
  

  $.post(presentationManager.settings.submitURL, data, function (data) {
  
    var result = presentation.parseJson(data);
    
    switch (result.status) {
      case 'error': {
        presentation.setError(result.message, 5000);
        break;
      }
      case 'success': {
        presentation.setStatus(result.message, 5000);
        break;
      }
      case 'reload': {
        presentation.setStatus(result.message);
        window.location = presentationManager.settings.reloadURL;
        break;
      }
    
    
    }
    
    
    
    
    
    presentationManager.deleteRows();
    presentationManager.rebuildTable();
    
    //Clear the delete object.
    presentationManager.toDelete = {};
    
  }); 
  
}




/***************************** Focus **********************************/


/**
 * Indicate which picture is selected.
 *
 */
presentationManager.focus = function (current) {
  var index = presentationManager.pictures[current];
   
  presentationManager.current = parseInt(index);

  $('.presentation-thumbnail-field').css({border: 'none', margin: '2px'});
  $('#presentation-thumbnail-field-' + current).css({border: '2px solid black', margin: '0'});

}






/**************************** Actions *********************************/


/**
 * Moves the row one slot up, unless the row is already at the very top.
 *
 */
presentationManager.moveUp = function (pid) {
  var index = presentationManager.pictures[pid];
  
  
  //Check to see if we're at the top.
  if (index == 0) {
    presentation.setError('Image already on top.');
    return;
  }
  

  presentationManager.switchRows(index, index - 1);

}


/**
 * Moves the row one slot down, unless the row is already at the very bottom.
 *
 */
presentationManager.moveDown = function (pid) {
  var index = presentationManager.pictures[pid];
  var length = presentationManager.index.length;
  
  //Check to see if we're at the bottom.
  if (index == length - 1) {
    presentation.setError('Image already at the bottom.');
    return;
  }
  

  presentationManager.switchRows(index, index + 1);

}


/**
 * Moves the row to the very top, unless the row is already at the top.
 *
 */
presentationManager.moveTop = function (pid) {
  var index = presentationManager.pictures[pid];

  //Check to see if we're at the top.
  if (index == 0) {
    presentation.setError('Image already on top.');
    return;
  }
  

  presentationManager.shiftTop(index);

}




/**
 * Moves the row to the very bottom, unless the row is already at the bottom.
 *
 */
presentationManager.moveBottom = function (pid) {
  var index = presentationManager.pictures[pid];
  
  //Check to see if we're at the bottom.
  if (index == presentationManager.count - 1) {
    presentation.setError('Image already at the bottom.');
    return;
  }
  

  presentationManager.shiftBottom(index);

}


/*************************** Deletion *********************************/


/**
 * Queues up the picture for deletion.
 *
 */
presentationManager.flagDeletion = function (pid) {
  presentationManager.toDelete[pid] = true;
  
  //Clearly mark the row to be deleted.
  $('#presentation-management-row-' + pid).css('background-color', '#FFCCCC'); 

}


/**
 * Remove a picture from the delete queue.
 *
 */
presentationManager.unflagDeletion = function (pid) {
  delete presentationManager.toDelete[pid];
  
  //Clearly mark the row to be deleted.
  $('#presentation-management-row-' + pid).css('background-color', ''); 

}


/**
 * Go through the deletion array and remove each pid from the main index.
 *
 */
presentationManager.deletePictures = function () {
  for (var pid in presentationManager.toDelete) {
    presentationManager.rebuildIndex(pid);
    presentationManager.count--;
  }
}



/**
 * Delete rows which contain pictures that were deleted.
 *
 */
presentationManager.deleteRows = function () {
  for (var pid in presentationManager.toDelete) {
    $('#presentation-management-row-' + pid).remove();
  }

}

 


/**
 * Rebuilds the index by realigning all the pictures with the indices.
 *
 * @param pid is the picture that should be left out.
 */
presentationManager.rebuildIndex = function (pid) {

  var index = presentationManager.pictures[pid];
  var length = presentationManager.index.length;
  
  for (var x = index; x < length - 1; x++) {

    presentationManager.index[x] = presentationManager.index[x + 1];
    presentationManager.pictures[presentationManager.index[x]] = x;
  }
  
  presentationManager.index = presentationManager.index.slice(0, length - 1);
  delete presentationManager.pictures[pid];
  
}



/**************************** Utilities *******************************/


/**
 * Get the ID of a picture using it's CSS id.
 *
 */
presentationManager.getID = function (css) {
  var index = css.lastIndexOf('-');

  return css.substring(index + 1); 
}




/**
 * Rebuild table zebra stripes.
 *
 */
presentationManager.rebuildTable = function () {
  var counter = 1;
  
  $('.presentation-management-row').each(function () {
    $(this).removeClass('odd').removeClass('even');
    
    if (counter % 2) {
      $(this).addClass('odd');
    }
    else {
      $(this).addClass('even');
    }
    
    counter++;
  });


}




/**
 * Switch two rows in the table. No validity checks performed.
 * 
 */
presentationManager.switchRows = function (source, destination) {
  
  var sourcePID = presentationManager.index[source];
  var destPID = presentationManager.index[destination];
  
  //Switch the picture with the picture in the destination index.  
  presentationManager.pictures[destPID] = source;
  presentationManager.pictures[sourcePID] = destination;
  
  presentationManager.index[source] = destPID;
  presentationManager.index[destination] = sourcePID;
  
  
  var sourceRow = $('#presentation-management-row-' + sourcePID);
  var destRow = $('#presentation-management-row-' + destPID);
  
  //Switch the actual DOM elements.
  if (destination > source) {
    $(sourceRow).insertAfter(destRow);  
  }
  else {
    $(sourceRow).insertBefore(destRow);
  }
    

  
  //Switch the coloring.
  $(sourceRow).toggleClass('odd').toggleClass('even');
  $(destRow).toggleClass('odd').toggleClass('even');  

  //Restripe.
  presentationManager.rebuildTable();

}


/**
 * Switch a row to the very top.
 * 
 */
presentationManager.shiftTop = function (source, destination) {
  
  var sourcePID = presentationManager.index[source];
  var destPID = presentationManager.index[0];
  
  var tempArray = new Array();
  
  presentationManager.rebuildIndex(sourcePID);

  for (var x in presentationManager.index) {
    var index = parseInt(x);
    
    presentationManager.pictures[presentationManager.index[index]] = index + 1; 
    tempArray[index + 1] = presentationManager.index[x];
  }
  
  tempArray[0] = sourcePID;
  presentationManager.pictures[sourcePID] = 0;
  presentationManager.index = tempArray;
  
  var sourceRow = $('#presentation-management-row-' + sourcePID);
  var destRow = $('#presentation-management-row-' + destPID);
  
  
  //Switch the actual DOM elements.
  $(sourceRow).insertBefore(destRow);

    
  //Restripe.
  presentationManager.rebuildTable();

}


/**
 * Switch a row to the very bottom.
 * 
 */
presentationManager.shiftBottom = function (source) {
  
  var sourcePID = presentationManager.index[source];
  var destPID = presentationManager.index[presentationManager.count - 1];
  
  presentationManager.rebuildIndex(sourcePID);
  
  presentationManager.pictures[sourcePID] = presentationManager.count - 1;
  presentationManager.index[presentationManager.count - 1] = sourcePID;
  

  var sourceRow = $('#presentation-management-row-' + sourcePID);
  var destRow = $('#presentation-management-row-' + destPID);
  
  //Switch the actual DOM elements.
  $(sourceRow).insertAfter(destRow);

    
  //Restripe.
  presentationManager.rebuildTable();
}






